@@ -2,28 +2,28 @@ use cargo::util::command_prelude::ArgMatchesExt;
 use cargo::CliResult;
 use cargo::Config;
 
-use cargo_c::build::{cbuild, config_configure};
+use cargo_c::build::cbuild;
+use cargo_c::cli::run_cargo_fallback;
 use cargo_c::cli::subcommand_install;
+use cargo_c::config::config_configure;
 use cargo_c::install::cinstall;
 
-use structopt::clap::*;
-
 fn main() -> CliResult {
     let mut config = Config::default()?;
 
     let subcommand = subcommand_install("cinstall", "Install the crate C-API");
-    let mut app = app_from_crate!()
-        .settings(&[
-            AppSettings::UnifiedHelpMessage,
-            AppSettings::DeriveDisplayOrder,
-            AppSettings::VersionlessSubcommands,
-        ])
+    let mut app = clap::command!()
+        .dont_collapse_args_in_usage(true)
+        .allow_external_subcommands(true)
         .subcommand(subcommand);
 
     let args = app.clone().get_matches();
 
     let subcommand_args = match args.subcommand() {
-        ("cinstall", Some(args)) => args,
+        Some(("cinstall", args)) => args,
+        Some((cmd, args)) => {
+            return run_cargo_fallback(cmd, args);
+        }
         _ => {
             // No subcommand provided.
             app.print_help()?;
@@ -31,7 +31,7 @@ fn main() -> CliResult {
         }
     };
 
-    if subcommand_args.is_present("version") {
+    if subcommand_args.flag("version") {
         println!("{} {}", env!("CARGO_PKG_NAME"), env!("CARGO_PKG_VERSION"));
         return Ok(());
     }
@@ -40,10 +40,9 @@ fn main() -> CliResult {
 
     let mut ws = subcommand_args.workspace(&config)?;
 
-    let (build_targets, install_paths, capi_config, _, _) =
-        cbuild(&mut ws, &config, &subcommand_args, "release")?;
+    let (packages, _) = cbuild(&mut ws, &config, subcommand_args, "release")?;
 
-    cinstall(&ws, &capi_config, build_targets, install_paths)?;
+    cinstall(&ws, &packages)?;
 
     Ok(())
 }
