@@ -2,7 +2,32 @@
 
 use crate::build::CApiConfig;
 use crate::install::InstallPaths;
-use std::path::{Path, PathBuf};
+use std::path::{Component, Path, PathBuf};
+
+fn canonicalize<P: AsRef<Path>>(path: P) -> String {
+    let mut separator = "";
+    let out = path
+        .as_ref()
+        .components()
+        .map(|p| match p {
+            Component::RootDir => {
+                separator = "/";
+                String::new()
+            }
+            Component::Prefix(_) => p.as_os_str().to_string_lossy().to_string(),
+            _ => {
+                let c = format!("{}{}", separator, p.as_os_str().to_string_lossy());
+                separator = "/";
+                c
+            }
+        })
+        .collect::<String>();
+    if out.is_empty() {
+        "/".to_string()
+    } else {
+        out
+    }
+}
 
 #[derive(Debug, Clone)]
 pub struct PkgConfig {
@@ -41,7 +66,7 @@ impl PkgConfig {
     /// Cflags: -I${includedir}/$name
     /// Libs: -L${libdir} -l$name
     ///
-    pub fn new(name: &str, capi_config: &CApiConfig) -> Self {
+    pub fn new(_name: &str, capi_config: &CApiConfig) -> Self {
         let requires = match &capi_config.pkg_config.requires {
             Some(reqs) => reqs.split(',').map(|s| s.trim().to_string()).collect(),
             _ => Vec::new(),
@@ -63,11 +88,13 @@ impl PkgConfig {
         ];
 
         let cflags = if capi_config.header.enabled {
-            if capi_config.header.subdirectory {
-                format!("-I{}/{}", "${includedir}", name)
-            } else {
-                String::from("-I${includedir}")
-            }
+            let includedir = Path::new("${includedir}").join(&capi_config.header.subdirectory);
+            let includedir = includedir
+                .ancestors()
+                .nth(capi_config.pkg_config.strip_include_path_components)
+                .unwrap_or_else(|| Path::new(""));
+
+            format!("-I{}", canonicalize(includedir))
         } else {
             String::from("")
         };
@@ -97,18 +124,30 @@ impl PkgConfig {
     pub(crate) fn from_workspace(
         name: &str,
         install_paths: &InstallPaths,
-        args: &structopt::clap::ArgMatches<'_>,
+        args: &clap::ArgMatches,
         capi_config: &CApiConfig,
     ) -> Self {
         let mut pc = PkgConfig::new(name, capi_config);
 
         pc.prefix = install_paths.prefix.clone();
         // TODO: support exec_prefix
-        if args.is_present("includedir") {
-            pc.includedir = install_paths.includedir.clone();
+        if args.contains_id("includedir") {
+            if let Ok(suffix) = install_paths.includedir.strip_prefix(&pc.prefix) {
+                let mut includedir = PathBuf::from("${prefix}");
+                includedir.push(suffix);
+                pc.includedir = includedir;
+            } else {
+                pc.includedir = install_paths.includedir.clone();
+            }
         }
-        if args.is_present("libdir") {
-            pc.libdir = install_paths.libdir.clone();
+        if args.contains_id("libdir") {
+            if let Ok(suffix) = install_paths.libdir.strip_prefix(&pc.prefix) {
+                let mut libdir = PathBuf::from("${prefix}");
+                libdir.push(suffix);
+                pc.libdir = libdir;
+            } else {
+                pc.libdir = install_paths.libdir.clone();
+            }
         }
         pc
     }
@@ -116,14 +155,10 @@ impl PkgConfig {
     pub(crate) fn uninstalled(&self, output: &Path) -> Self {
         let mut uninstalled = self.clone();
         uninstalled.prefix = output.to_path_buf();
-        uninstalled.includedir = "${prefix}".into();
+        uninstalled.includedir = "${prefix}/include".into();
         uninstalled.libdir = "${prefix}".into();
         // First libs item is the search path
         uninstalled.libs[0] = "-L${prefix}".into();
-        // First cflags item is the in include dir, if lib provides headers
-        if uninstalled.cflags[0].starts_with("-I") {
-            uninstalled.cflags[0] = "-I{prefix}".into();
-        }
 
         uninstalled
     }
@@ -184,39 +219,40 @@ Description: {}
 Version: {}
 Libs: {}
 Cflags: {}",
-            self.prefix.display(),
-            self.exec_prefix.display(),
-            self.libdir.display(),
-            self.includedir.display(),
+            canonicalize(&self.prefix),
+            canonicalize(&self.exec_prefix),
+            canonicalize(&self.libdir),
+            canonicalize(&self.includedir),
             self.name,
-            self.description,
+            // avoid endlines
+            self.description.replace('\n', " "),
             self.version,
             self.libs.join(" "),
             self.cflags.join(" "),
         );
 
         if !self.libs_private.is_empty() {
-            base.push_str(&format!(
+            base.push_str(
                 "
-Libs.private: {}",
-                self.libs_private.join(" "),
-            ));
+Libs.private: ",
+            );
+            base.push_str(&self.libs_private.join(" "));
         }
 
         if !self.requires.is_empty() {
-            base.push_str(&format!(
+            base.push_str(
                 "
-Requires: {}",
-                self.requires.join(", ")
-            ));
+Requires: ",
+            );
+            base.push_str(&self.requires.join(", "));
         }
 
         if !self.requires_private.is_empty() {
-            base.push_str(&format!(
+            base.push_str(
                 "
-Requires.private: {}",
-                self.requires_private.join(", ")
-            ));
+Requires.private: ",
+            );
+            base.push_str(&self.requires_private.join(", "));
         }
 
         /*
@@ -244,25 +280,28 @@ mod test {
             &CApiConfig {
                 header: crate::build::HeaderCApiConfig {
                     name: "foo".into(),
-                    subdirectory: true,
+                    subdirectory: "".into(),
                     generation: true,
                     enabled: true,
                 },
                 pkg_config: crate::build::PkgConfigCApiConfig {
                     name: "foo".into(),
+                    filename: "foo".into(),
                     description: "".into(),
                     version: "0.1".into(),
                     requires: Some("somelib, someotherlib".into()),
                     requires_private: Some("someprivatelib >= 1.0".into()),
+                    strip_include_path_components: 0,
                 },
                 library: crate::build::LibraryCApiConfig {
                     name: "foo".into(),
                     version: Version::parse("0.1.0").unwrap(),
                     install_subdir: None,
                     versioning: true,
-                    rustflags: String::default(),
-                    original_rustflags: String::default(),
+                    import_library: true,
+                    rustflags: Vec::default(),
                 },
+                install: Default::default(),
             },
         );
         pkg.add_lib("-lbar").add_cflag("-DFOO");
