@@ -70,23 +70,50 @@ impl Target {
         let env = &self.env;
 
         if os == "android" {
-            lines.push(format!("-Wl,-soname,lib{}.so", lib_name));
-        } else if env != "musl"
-            && (os == "linux" || os == "freebsd" || os == "dragonfly" || os == "netbsd")
+            lines.push(format!("-Wl,-soname,lib{lib_name}.so"));
+        } else if os == "linux"
+            || os == "freebsd"
+            || os == "dragonfly"
+            || os == "netbsd"
+            || os == "haiku"
+            || os == "illumos"
         {
-            lines.push(format!("-Wl,-soname,lib{}.so.{}", lib_name, major));
+            if capi_config.library.versioning {
+                lines.push(format!("-Wl,-soname,lib{lib_name}.so.{major}"));
+            } else {
+                lines.push(format!("-Wl,-soname,lib{lib_name}.so"));
+            }
         } else if os == "macos" || os == "ios" {
-            let line = format!("-Wl,-install_name,{1}/lib{0}.{2}.{3}.{4}.dylib,-current_version,{2}.{3}.{4},-compatibility_version,{2}",
-                    lib_name, libdir.display(), major, minor, patch);
-            lines.push(line)
+            let line = if capi_config.library.versioning {
+                let install_ver = if major == 0 {
+                    format!("{major}.{minor}")
+                } else {
+                    format!("{major}")
+                };
+                format!("-Wl,-install_name,{1}/lib{0}.{5}.dylib,-current_version,{2}.{3}.{4},-compatibility_version,{5}",
+                        lib_name, libdir.display(), major, minor, patch, install_ver)
+            } else {
+                format!(
+                    "-Wl,-install_name,{1}/lib{0}.dylib",
+                    lib_name,
+                    libdir.display()
+                )
+            };
+            lines.push(line);
+            // Enable larger LC_RPATH and install_name entries
+            lines.push("-Wl,-headerpad_max_install_names".to_string());
         } else if os == "windows" && env == "gnu" {
             // This is only set up to work on GNU toolchain versions of Rust
             lines.push(format!(
                 "-Wl,--output-def,{}",
-                target_dir.join(format!("{}.def", lib_name)).display()
+                target_dir.join(format!("{lib_name}.def")).display()
             ));
         }
 
+        // Emscripten doesn't support soname or other dynamic linking flags (yet).
+        // See: https://github.com/emscripten-core/emscripten/blob/3.1.39/emcc.py#L92-L94
+        // else if os == "emscripten"
+
         lines
     }
 }
